import { Observable } from "iterable-observer";
export type Middleware<C = any> = (context: C, next: () => Promise<any>) => Promise<any> | any;
export class Stack<C = any> {
    length: number;
    use(...middlewares: Middleware<C>[]): this;
    execute(context?: C, depth?: number): any;
    mount(condition: (context: C) => boolean, stack: Stack<C>): this;
}
export function parseDocument(response: Response): Promise<Document>;
export function makeFormData(data: Record<string, any>): FormData;
export function serializeNode(root: Node): {
    contentType: string;
    data: string;
} | {
    contentType: string;
    data: FormData;
};
export function serialize(data: any, contentType?: string): {
    data: any;
    contentType: string;
};
enum FileMethod {
    text = "readAsText",
    dataURL = "readAsDataURL",
    binaryString = "readAsBinaryString",
    arrayBuffer = "readAsArrayBuffer"
}
export function readAs(file: Blob, method: keyof typeof FileMethod, encoding?: string): {
    progress: Observable<ProgressEvent<EventTarget>>;
    result: Promise<string | ArrayBuffer>;
};
export enum BodyRequestMethods {
    POST = "POST",
    PUT = "PUT",
    PATCH = "PATCH",
    DELETE = "DELETE"
}
export interface RequestOptions {
    withCredentials?: boolean;
    timeout?: number;
    responseType?: XMLHttpRequestResponseType;
}
export interface Request extends RequestOptions {
    method?: 'HEAD' | 'GET' | keyof typeof BodyRequestMethods;
    path: string | URL;
    headers?: HeadersInit;
    body?: BodyInit | HTMLFormElement | any;
}
export interface Response<B = Request['body']> {
    status: number;
    statusText: string;
    headers: Record<string, string | object>;
    body?: B;
}
export class HTTPError<B = Request['body']> extends URIError {
    status: number;
    statusText: string;
    headers: Response<B>['headers'];
    body?: B;
    constructor(message: string, response: Response);
}
export type LinkHeader = Record<string, {
    URI: string;
    rel: string;
    title?: string;
}>;
export const headerParser: {
    Link(value: string): LinkHeader;
};
export function parseHeaders(raw: string): Response['headers'];
export interface RequestResult<B> {
    response: Promise<Response<B>>;
    upload?: Observable<ProgressEvent>;
    download?: Observable<ProgressEvent>;
}
export function requestXHR<B>({ method, path, headers, body, ...rest }: Request): RequestResult<B>;
export function requestFetch<B>({ path, method, headers, withCredentials, body, timeout, responseType }: Request): Promise<Response<B>>;
export function request<B>(options: Request): RequestResult<B>;
export interface Context {
    request: Request;
    response: Response;
}
export interface ClientOptions extends RequestOptions {
    baseURI?: string;
}
export class HTTPClient<T extends Context> extends Stack<T> {
    baseURI: string;
    options: RequestOptions;
    constructor({ baseURI, ...options }?: ClientOptions);
    defaultWare: Middleware<T>;
    use(...middlewares: Middleware<T>[]): this;
    request<B>(data: T['request']): Promise<Response<B>>;
    head(path: Request['path'], headers?: Request['headers']): Promise<Record<string, string | object>>;
    get<B>(path: Request['path'], headers?: Request['headers']): Promise<Response<B>>;
    post<B>(path: Request['path'], body?: Request['body'], headers?: Request['headers']): Promise<Response<B>>;
    put<B>(path: Request['path'], body?: Request['body'], headers?: Request['headers']): Promise<Response<B>>;
    patch<B>(path: Request['path'], body?: Request['body'], headers?: Request['headers']): Promise<Response<B>>;
    delete<B>(path: Request['path'], body?: Request['body'], headers?: Request['headers']): Promise<Response<B>>;
}

//# sourceMappingURL=index.d.ts.map
